<?php
/* --------------------------------------------------------------
	FullyQualifiedProductID.php 2014-03-12_1546 mabr
	Gambio GmbH
	http://www.gambio.de
	Copyright (c) 2013 Gambio GmbH
	Released under the GNU General Public License (Version 2)
	[http://www.gnu.org/licenses/gpl-2.0.html]
	--------------------------------------------------------------
*/

/*
Product IDs may be:

- simple
42
representing a simple product without options, properties or GX Customizer data

- with options
42{3}5{18}7
42|3-5_18-7
consisting of the products_id and one or more pairs of options_id and options_values_id

- with properties
42x23
42|23
products_id and properties_combis_id

- with options and properties
42{3}5{4}7{28}3x23
42|3-5_4-7_28-3|23

- with GX Customizer Data
42{138478}0
(used internally only)

The first representation (with braces and x) is mostly used internally, e.g. in the shopping cart.
The second represention is used whenever products need to be referenced with a fully qualified ID when talking 
to external systems as braces might cause problems in these cases.
*/

class FullyQualifiedProductID {
	const TYPE_SIMPLE = 0x00;
	const TYPE_OPTIONS = 0x01;
	const TYPE_PROPERTIES = 0x02;
	const TYPE_OPTIONSPROPERTIES = 0x03; // note that TYPE_OPTIONSPROPERTIES == TYPE_OPTIONS|TYPE_PROPERTIES
	const TYPE_CUSTOMIZER = 0x04;

	public $type;
	public $products_id;
	public $options;
	public $properties_combis_id;
	public $customizer_id;

	public function __construct($type_or_string = null, $products_id = null, $options = null, $properties_combis_id = null, $customizer_id = null) {
		if(is_string($type_or_string)) {
			$this->setDataFromString($type_or_string);
		}
		else {
			$this->type = $type;
			$this->products_id = $products_id;
			$this->options = $options;
			$this->properties_combis_id = $properties_combis_id;
			$this->customizer_id = $customizer_id;
		}
	}

	public function setDataFromString($string) {
		$this->decodeFullyQualifiedProductID($string);
	}

	public function addOption($options_id, $options_values_id) {
		$this->options[$options_id] = $options_values_id;
	}

	public function isValid() {
		if(empty($this->products_id)) {
			echo "no products_id\n";
			var_dump($this);
			return false;
		}
		if($this->_productHasProperties() && empty($this->properties_combis_id)) {
			echo "no combis_id\n";
			return false;
		}
		if($this->_productHasOptions() && empty($this->options)) {
			echo "no options\n";
			return false;
		}
		return true;
	}

	protected function _productHasOptions() {
		$query = "SELECT COUNT(*) AS `num_options` FROM `products_attributes` WHERE `products_id` = :products_id";
		$query = strtr($query, array(':products_id' => $this->products_id));
		$result = xtc_db_query($query);
		$has_options = false;
		while($row = xtc_db_fetch_array($result)) {
			$has_options = $row['num_options'] > 0;
		}
		return $has_options;
	}

	protected function _productHasProperties() {
		$query = "SELECT COUNT(*) AS `num_properties` FROM `products_properties_combis` WHERE `products_id` = :products_id";
		$query = strtr($query, array(':products_id' => $this->products_id));
		$result = xtc_db_query($query);
		$has_properties = false;
		while($row = xtc_db_fetch_array($result)) {
			$has_properties = $row['num_properties'] > 0;
		}
		return $has_properties;
	}

	public function toInternalRepresentation() {
		$int_string = $this->products_id;
		if(!empty($this->options)) {
			foreach($this->options as $options_id => $options_values_id) {
				$int_string .= "{".$options_id."}$options_values_id";
			}
		}
		if(empty($this->customizer_id) !== true)
		{
			$int_string .= '{'.$this->customizer_id.'}0';
		}
		if(!empty($this->properties_combis_id)) {
			$int_string .= "x".$this->properties_combis_id;
		}
		return $int_string;
	}

	public function toExternalRepresentation() {
		$ext_string = $this->products_id;
		if(!empty($this->options)) {
			$ext_string .= '|';
			$options_parts = array();
			foreach($this->options as $options_id => $options_values_id) {
				$options_parts[] = $options_id.'-'.$options_values_id;
			}
			$ext_string .= implode('_', $options_parts);
		}
		if(!empty($this->properties_combis_id)) {
			$ext_string .= '|'.$this->properties_combis_id;
		}
		return $ext_string;
	}

	public function __toString() {
		$string = $this->toExternalRepresentation();
		if(!is_string($string)) {
			if(is_numeric($string)) {
				$string = (string)$string;
			}
			else {
				throw new Exception('internal error (fqpid representation)');
			}
		}
		return $string;
	}

	public function decodeFullyQualifiedProductID($fqpid) {
		$matches = array();
		if(preg_match('/[{}|x]/', $fqpid) == 0) {
			$this->type = FullyQualifiedProductID::TYPE_SIMPLE;
			$this->products_id = (int)$fqpid;
			$this->options = null;
			$this->properties_combis_id = null;
		}
		else if(preg_match('/^(\d+)[|x](\d+)$/', $fqpid, $matches) == 1) {  // properties (int and ext)
			$this->type = FullyQualifiedProductID::TYPE_PROPERTIES;
			$this->products_id = (int)$matches[1];
			$this->options = null;
			$this->properties_combis_id = (int)$matches[2];
		}
		else if(preg_match('/^(\d+)((\{\d+\}\d+)+)$/', $fqpid, $matches) == 1) {   // options (int)
			$this->type = FullyQualifiedProductID::TYPE_OPTIONS;
			$this->products_id = (int)$matches[1];
			$this->options = $this->_splitInternalOptions($matches[2]);
			$this->properties_combis_id = null;
		}
		else if(preg_match('/^(\d+)\|(\d+-\d+(_\d+-\d+)*)$/', $fqpid, $matches) == 1) {  // options (ext)
			$this->type = FullyQualifiedProductID::TYPE_OPTIONS;
			$this->products_id = (int)$matches[1];
			$this->options = $this->_splitExternalOptions($matches[2]);
			$this->properties_combis_id = null;
		}
		else if(preg_match('/^(\d+)((\{\d+\}\d+)+)x(\d+)$/', $fqpid, $matches) == 1) {  // options and properties (int)
			$this->type = FullyQualifiedProductID::TYPE_OPTIONSPROPERTIES;
			$this->products_id = (int)$matches[1];
			$this->options = $this->_splitInternalOptions($matches[2]);
			$this->properties_combis_id = (int)$matches[4];
		}
		else if(preg_match('/^(\d+)\|(\d+-\d+(_\d+-\d+))\|(\d+)?$/', $fqpid, $matches) == 1) {  // options and properties (ext)
			$this->type = FullyQualifiedProductID::TYPE_OPTIONSPROPERTIES;
			$this->products_id = (int)$matches[1];
			$this->options = $this->_splitExternalOptions($matches[2]);
			$this->properties_combis_id = (int)$matches[4];
		}
	}

	protected function _splitInternalOptions($options_string) {
		$parts = explode('{', $options_string);
		$options = array();
		foreach($parts as $p) {
			if(empty($p)) {
				continue;
			}
			list($options_id, $options_values_id) = explode('}', $p);
			if($options_values_id == 0) {
				$this->type |= self::TYPE_CUSTOMIZER;
				$this->customizer_id = $options_id;
			}
			else
			{
				$options[$options_id] = $options_values_id;
			}
		}
		return $options;
	}

	protected function _splitExternalOptions($options_string) {
		$parts = explode('_', $options_string);
		$options = array();
		foreach($parts as $p) {
			list($options_id, $options_values_id) = explode('-', $p);
			$options[$options_id] = $options_values_id;
		}
		return $options;
	}
}
